@@ -66,7 +66,6 @@ from airflow.serialization.serialized_objects import SerializedBaseOperator
 from airflow.timetables.base import DataInterval
 from airflow.utils.session import NEW_SESSION, provide_session
 from airflow.utils.state import DagRunState, State, TaskInstanceState
-from airflow.utils.task_group import TaskGroup
 from airflow.utils.timezone import coerce_datetime, datetime
 from airflow.utils.types import DagRunType
 
@@ -75,10 +74,18 @@ from tests_common.test_utils.mock_operators import MockOperator
 from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS
 
 if AIRFLOW_V_3_0_PLUS:
+    from airflow.models.dag_version import DagVersion
     from airflow.sdk import task as task_deco
     from airflow.utils.types import DagRunTriggeredByType
 else:
-    from airflow.decorators import task as task_deco
+    from airflow.decorators import task as task_deco  # type: ignore[attr-defined,no-redef]
+
+try:
+    from airflow.sdk.definitions.taskgroup import TaskGroup
+except ImportError:
+    # Fallback for Airflow < 3.1
+    from airflow.utils.task_group import TaskGroup  # type: ignore[no-redef]
+
 pytestmark = pytest.mark.db_test
 
 
@@ -121,10 +128,17 @@ class TestExternalTaskSensorV2:
             with TaskGroup(group_id=TEST_TASK_GROUP_ID) as task_group:
                 _ = [EmptyOperator(task_id=f"task{i}") for i in range(len(target_states))]
             dag.sync_to_db()
+        if AIRFLOW_V_3_0_PLUS:
+            SerializedDagModel.write_dag(dag, bundle_name="testing")
+        else:
             SerializedDagModel.write_dag(dag)
 
         for idx, task in enumerate(task_group):
-            ti = TaskInstance(task=task, run_id=self.dag_run_id)
+            if AIRFLOW_V_3_0_PLUS:
+                dag_version = DagVersion.get_latest_version(task_group[idx].dag_id)
+                ti = TaskInstance(task=task, run_id=self.dag_run_id, dag_version_id=dag_version.id)
+            else:
+                ti = TaskInstance(task=task, run_id=self.dag_run_id)
             ti.run(ignore_ti_state=True, mark_success=True)
             ti.set_state(target_states[idx])
 
@@ -144,16 +158,27 @@ class TestExternalTaskSensorV2:
                 fake_task()
                 fake_mapped_task.expand(x=list(map_indexes))
         dag.sync_to_db()
-        SerializedDagModel.write_dag(dag)
+        if AIRFLOW_V_3_0_PLUS:
+            SerializedDagModel.write_dag(dag, bundle_name="testing")
+        else:
+            SerializedDagModel.write_dag(dag)
 
         for task in task_group:
             if task.task_id == "fake_mapped_task":
                 for map_index in map_indexes:
-                    ti = TaskInstance(task=task, run_id=self.dag_run_id, map_index=map_index)
+                    if AIRFLOW_V_3_0_PLUS:
+                        dag_version = DagVersion.get_latest_version(dag.dag_id)
+                        ti = TaskInstance(task=task, run_id=self.dag_run_id, dag_version_id=dag_version.id)
+                    else:
+                        ti = TaskInstance(task=task, run_id=self.dag_run_id, map_index=map_index)
                     ti.run(ignore_ti_state=True, mark_success=True)
                     ti.set_state(target_state)
             else:
-                ti = TaskInstance(task=task, run_id=self.dag_run_id)
+                if AIRFLOW_V_3_0_PLUS:
+                    dag_version = DagVersion.get_latest_version(dag.dag_id)
+                    ti = TaskInstance(task=task, run_id=self.dag_run_id, dag_version_id=dag_version.id)
+                else:
+                    ti = TaskInstance(task=task, run_id=self.dag_run_id)
                 ti.run(ignore_ti_state=True, mark_success=True)
                 ti.set_state(target_state)
 
@@ -999,7 +1024,7 @@ exit 0
 
     @pytest.mark.execution_timeout(10)
     def test_external_task_sensor_deferrable(self, dag_maker):
-        context = {}
+        context = {"execution_date": DEFAULT_DATE}
         with dag_maker() as dag:
             op = ExternalTaskSensor(
                 task_id="test_external_task_sensor_check",
@@ -1022,6 +1047,30 @@ exit 0
         assert exc.value.trigger.external_task_ids == ["test_task"]
         assert exc.value.trigger.execution_dates == [DEFAULT_DATE]
 
+    def test_get_logical_date(self):
+        """For AF 2, we check for execution_date in context."""
+        context = {"execution_date": DEFAULT_DATE}
+        op = ExternalTaskSensor(
+            task_id="test_external_task_sensor_check",
+            external_dag_id="test_dag_parent",
+            external_task_id="test_task",
+        )
+        assert op._get_logical_date(context) == DEFAULT_DATE
+
+    def test_handle_execution_date_fn(self):
+        def func(dt, context):
+            assert context["execution_date"] == dt
+            return dt + timedelta(0)
+
+        op = ExternalTaskSensor(
+            task_id="test_external_task_sensor_check",
+            external_dag_id="test_dag_parent",
+            external_task_id="test_task",
+            execution_date_fn=func,
+        )
+        context = {"execution_date": DEFAULT_DATE}
+        assert op._handle_execution_date_fn(context) == DEFAULT_DATE
+
 
 @pytest.mark.skipif(not AIRFLOW_V_3_0_PLUS, reason="Different test for AF 2")
 @pytest.mark.usefixtures("testing_dag_bundle")
@@ -1310,6 +1359,43 @@ class TestExternalTaskSensorV3:
             task_group_id="test_group",
         )
 
+    def test_get_logical_date(self):
+        """For AF 3, we check for logical date or dag_run.run_after  in context."""
+
+        context = {"logical_date": DEFAULT_DATE}
+        op = ExternalTaskSensor(
+            task_id="test_external_task_sensor_check",
+            external_dag_id="test_dag_parent",
+            external_task_id="test_task",
+        )
+        assert op._get_logical_date(context) == DEFAULT_DATE
+
+    def test_get_logical_date_with_dag_run_after(self):
+        """For AF 3, we check for logical date or dag_run.run_after  in context."""
+        op = ExternalTaskSensor(
+            task_id="test_external_task_sensor_check",
+            external_dag_id="test_dag_parent",
+            external_task_id="test_task",
+        )
+        mock_dag_run = mock.MagicMock()
+        mock_dag_run.run_after = DEFAULT_DATE
+        context = {"dag_run": mock_dag_run}
+        assert op._get_logical_date(context) == DEFAULT_DATE
+
+    def test_handle_execution_date_fn(self):
+        def func(dt, context):
+            assert context["logical_date"] == dt
+            return dt + timedelta(0)
+
+        op = ExternalTaskSensor(
+            task_id="test_external_task_sensor_check",
+            external_dag_id="test_dag_parent",
+            external_task_id="test_task",
+            execution_date_fn=func,
+        )
+        context = {"logical_date": DEFAULT_DATE}
+        assert op._handle_execution_date_fn(context) == DEFAULT_DATE
+
 
 class TestExternalTaskAsyncSensor:
     TASK_ID = "external_task_sensor_check"
@@ -1847,6 +1933,8 @@ def dag_bag_cyclic():
 
         for dag in dags:
             if AIRFLOW_V_3_0_PLUS:
+                dag.sync_to_db()
+                SerializedDagModel.write_dag(dag, bundle_name="testing")
                 dag_bag.bag_dag(dag=dag)
             else:
                 dag_bag.bag_dag(dag=dag, root_dag=dag)  # type: ignore[call-arg]
@@ -2004,6 +2092,9 @@ def dag_bag_head_tail(session):
 @provide_session
 def test_clear_overlapping_external_task_marker(dag_bag_head_tail, session):
     dag: DAG = dag_bag_head_tail.get_dag("head_tail")
+    dag.sync_to_db()
+    if AIRFLOW_V_3_0_PLUS:
+        SerializedDagModel.write_dag(dag, bundle_name="testing")
 
     # "Run" 10 times.
     for delta in range(10):
@@ -2021,7 +2112,12 @@ def test_clear_overlapping_external_task_marker(dag_bag_head_tail, session):
             dagrun.execution_date = logical_date
         session.add(dagrun)
         for task in dag.tasks:
-            ti = TaskInstance(task=task)
+            if AIRFLOW_V_3_0_PLUS:
+                dag_version = DagVersion.get_latest_version(task.dag_id, session=session)
+                ti = TaskInstance(task=task, dag_version_id=dag_version.id)
+
+            else:
+                ti = TaskInstance(task=task)
             dagrun.task_instances.append(ti)
             ti.state = TaskInstanceState.SUCCESS
     session.flush()
@@ -2029,9 +2125,11 @@ def test_clear_overlapping_external_task_marker(dag_bag_head_tail, session):
     assert dag.clear(start_date=DEFAULT_DATE, dag_bag=dag_bag_head_tail, session=session) == 30
 
 
-@provide_session
 def test_clear_overlapping_external_task_marker_with_end_date(dag_bag_head_tail, session):
     dag: DAG = dag_bag_head_tail.get_dag("head_tail")
+    dag.sync_to_db()
+    if AIRFLOW_V_3_0_PLUS:
+        SerializedDagModel.write_dag(dag=dag, bundle_name="testing")
 
     # "Run" 10 times.
     for delta in range(10):
@@ -2048,8 +2146,13 @@ def test_clear_overlapping_external_task_marker_with_end_date(dag_bag_head_tail,
         else:
             dagrun.execution_date = logical_date
         session.add(dagrun)
+
         for task in dag.tasks:
-            ti = TaskInstance(task=task)
+            if AIRFLOW_V_3_0_PLUS:
+                dag_version = DagVersion.get_latest_version(dag.dag_id, session=session)
+                ti = TaskInstance(task=task, dag_version_id=dag_version.id)
+            else:
+                ti = TaskInstance(task=task)
             dagrun.task_instances.append(ti)
             ti.state = TaskInstanceState.SUCCESS
     session.flush()
@@ -2108,6 +2211,7 @@ def dag_bag_head_tail_mapped_tasks():
         head >> body >> tail
 
     if AIRFLOW_V_3_0_PLUS:
+        dag.sync_to_db()
         dag_bag.bag_dag(dag=dag)
     else:
         dag_bag.bag_dag(dag=dag, root_dag=dag)
@@ -2115,10 +2219,11 @@ def dag_bag_head_tail_mapped_tasks():
     return dag_bag
 
 
-@provide_session
 def test_clear_overlapping_external_task_marker_mapped_tasks(dag_bag_head_tail_mapped_tasks, session):
     dag: DAG = dag_bag_head_tail_mapped_tasks.get_dag("head_tail")
-
+    dag.sync_to_db()
+    if AIRFLOW_V_3_0_PLUS:
+        SerializedDagModel.write_dag(dag=dag, bundle_name="testing")
     # "Run" 10 times.
     for delta in range(10):
         logical_date = DEFAULT_DATE + timedelta(days=delta)
@@ -2137,11 +2242,24 @@ def test_clear_overlapping_external_task_marker_mapped_tasks(dag_bag_head_tail_m
         for task in dag.tasks:
             if task.task_id == "dummy_task":
                 for map_index in range(5):
-                    ti = TaskInstance(task=task, run_id=dagrun.run_id, map_index=map_index)
+                    if AIRFLOW_V_3_0_PLUS:
+                        dag_version = DagVersion.get_latest_version(dag.dag_id, session=session)
+                        ti = TaskInstance(
+                            task=task,
+                            run_id=dagrun.run_id,
+                            map_index=map_index,
+                            dag_version_id=dag_version.id,
+                        )
+                    else:
+                        ti = TaskInstance(task=task, run_id=dagrun.run_id, map_index=map_index)
                     ti.state = TaskInstanceState.SUCCESS
                     dagrun.task_instances.append(ti)
             else:
-                ti = TaskInstance(task=task, run_id=dagrun.run_id)
+                if AIRFLOW_V_3_0_PLUS:
+                    dag_version = DagVersion.get_latest_version(dag.dag_id, session=session)
+                    ti = TaskInstance(task=task, run_id=dagrun.run_id, dag_version_id=dag_version.id)
+                else:
+                    ti = TaskInstance(task=task, run_id=dagrun.run_id)
                 ti.state = TaskInstanceState.SUCCESS
                 dagrun.task_instances.append(ti)
     session.flush()
